/*
 *  Copyright (C) 1998,9 by Marco G"otze.
 *
 *  This code is part of the wmpinboard source package, which is
 *  distributed under the terms of the GNU GPL2.
 */

#include <stdlib.h>
#include <stdio.h>

#include "wmpinboard.h"
#include "xmisc.h"
#include "notes.h"

extern data_t ndata[MAX_NOTES];
extern int notes_count;

extern XpmIcon app;
extern GC fontGC;

/*
 * adds a new note and returns its number (or -1 if notes limit exceeded)
 */
int
add_note()
{
  int cols[C_NUM];
  int rep = 20;
  int i, j, n;

  if (notes_count >= MAX_NOTES)
    return -1;
  n = notes_count++;

  /* find a reasonable color */
  for (i = 0; i < C_NUM; i++) cols[i] = 1;
  for (i = 0; i < n; i++) cols[ndata[i].col] = 0;
  for (i = j = 0; i < C_NUM; i++)
    if (cols[i]) cols[j++] = i;
  ndata[n].col = j ? cols[rand() % j] : rand() % C_NUM;

  /* choose a random place to pin it, some pixels off any other note's origin
     (possible if MAX_NOTES is sufficiently small) */
  do {
    ndata[n].x =  6 + rand() % 36;
    ndata[n].y = 12 + rand() % 34;  /* avoid "TO DO" label */
    j = 1;
    for (i = 0; i < n; i++)
      if (ndata[i].x - ndata[n].x < 5 && ndata[i].x - ndata[n].x > -5 &&
          ndata[i].y - ndata[n].y < 5 && ndata[i].y - ndata[n].y > -5)
      {
        j = 0;
        break;
      }
  } while (!j && --rep);

  /* initialize the note's text with blanks */
  for (i = 0; i < 59; i++) ndata[n].text[i] = ' ';
  ndata[n].text[59] = '\0';

  return n;
}

/*
 * removes a note
 */
void
remove_note(int number)
{
  int i;

  if (number >= notes_count)
    return;
  for (i = number; i < notes_count-1; i++)
    memcpy(&ndata[i], &ndata[i+1], sizeof(data_t));
  notes_count--;
}

/*
 * moves a note on top of all others and returns the quote's new ID
 */
int
raise_note(int number)
{
  data_t tmp;
  int i;

  if (number >= notes_count-1 || number < 0)
    return number;
  memcpy(&tmp, &ndata[number], sizeof(data_t));
  for (i = number; i < notes_count-1; i++)
    memcpy(&ndata[i], &ndata[i+1], sizeof(data_t));
  memcpy(&ndata[notes_count-1], &tmp, sizeof(data_t));
  return notes_count-1;
}

/*
 * returns a value corresponding to how tilted a note is to be displayed
 */
int
note_tilt(int note)
{
  return ndata[note].x < 14 ? 0 : (ndata[note].x < 36 ? 1 : 2);
}

/*
 * returns a true value if a text string is to be considered empty
 * (if width > 0, limit the check to width characters)
 */
int
string_empty(char *ptr, int width)
{
  int i = 0;

  while (*ptr && (!width || i++ < width))
    if (*ptr++ != ' ') return 0;
  return 1;
}

/*
 * creates templates for the icons suiting the specified note
 */
void
color_notes(int note)
{
  replace_color(board_img, 128, 0, 16, 48, XGetPixel(board_img, 136, 24),
    get_color(ndata[note].col));
}

/*
 * draws a note's miniature representation on the pinboard
 */
void
pin_note(int note)
{
  copy_image_area(board_img, board_img, 64, 0, 0, 0, 64, 64);
  merge_masked(board_img, board_img, 128, 16*note_tilt(note), ndata[note].x,
    ndata[note].y, 16, 16, C_OUTER);
}

/*
 * renders the pinboard with all of its notes excluding exclude if >= 0
 */
void
render_pinboard(int exclude)
{
  int i;

  XGetSubImage(display, app.pixmap, 64, 0, 64, 64, ~0, ZPixmap, board_img,
    64, 0);
  for (i = 0; i < notes_count; i++)
    if (i != exclude) {
      color_notes(i);
      merge_masked(board_img, board_img, 128, 16*note_tilt(i), 64+ndata[i].x,
        ndata[i].y, 16, 16, C_OUTER);
    }
  copy_image_area(board_img, board_img, 64, 0, 0, 0, 64, 64);
}

/*
 * if (x,y) is within a quote's area, returns the quote's ID, otherwise -1
 */
int
selected_note(int x, int y)
{
  unsigned long c = get_color(C_OUTER);
  int i;

  for (i = notes_count-1; i >= 0; i--)
    if (x >= ndata[i].x && x < ndata[i].x+16 &&
        y >= ndata[i].y && y < ndata[i].y+16 &&
        XGetPixel(board_img, 128+x-ndata[i].x, note_tilt(i)*16+y-ndata[i].y)
          != c)
    {
      return i;
    }
  return -1;
}

/*
 * prints a letter identified by a note and a position in its text string
 */
void
print_letter(int note, int letter)
{
  copy_image_area(char_img, note_img,
    (((unsigned char) ndata[note].text[letter])-1)*6, 0, 2+6*(letter%10),
    2+10*(letter/10), 6, 10);
}

/*
 * prints a note's entire text
 */
void
print_text(int note)
{
  int i;

  for (i = 0; i < 59; i++)
    print_letter(note, i);
}

/*
 * colors an edit mode note
 */
void
color_note(int note)
{
  copy_image_area(note_img, note_img, 64, 0, 0, 0, 64, 64);
  replace_color(note_img, 0, 0, 64, 64, get_color(C_INNER),
    get_color(ndata[note].col));
  replace_color(note_img, 55, 55, 8, 8, get_color(C_OUTER),
    get_text_color(ndata[note].col));
}

/*
 * shifts part of a note's text to the right, inserting ch (step >= 1)
 */
void
shift_string(int note, int at, int margin, int step, int ch)
{
  int i;

  if (step < 1)
    return;
  for (i = margin; i >= at+step; i--) {
    ndata[note].text[i] = ndata[note].text[i-step];
    print_letter(note, i);
  }
  for (i = at; i < at+step; i++) {
    ndata[note].text[i] = ch;
    print_letter(note, i);
  }
}

